@@ -1,4 +1,5 @@
 import re
+from contextlib import contextmanager
 
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.functional import cached_property
@@ -13,16 +14,16 @@ except ImportError as err:
         'Did you install psycopg2 or psycopg2-binary?'
     ) from err
 
-from django.db.backends.postgresql.base import (    # isort:skip
+from django.db.backends.postgresql.base import (
     DatabaseWrapper as PostgresDatabaseWrapper,
 )
 
-from .client import DatabaseClient                  # isort:skip
-from .creation import DatabaseCreation              # isort:skip
-from .features import DatabaseFeatures              # isort:skip
-from .introspection import DatabaseIntrospection    # isort:skip
-from .operations import DatabaseOperations          # isort:skip
-from .schema import DatabaseSchemaEditor            # isort:skip
+from .client import DatabaseClient
+from .creation import DatabaseCreation
+from .features import DatabaseFeatures
+from .introspection import DatabaseIntrospection
+from .operations import DatabaseOperations
+from .schema import DatabaseSchemaEditor
 
 
 class DatabaseWrapper(PostgresDatabaseWrapper):
@@ -69,6 +70,14 @@ class DatabaseWrapper(PostgresDatabaseWrapper):
         with self.wrap_database_errors:
             self.connection.autocommit = autocommit
 
+    @contextmanager
+    def _nodb_cursor(self):
+        # Overidden to avoid inapplicable "Django was unable to create a
+        # connection to the 'postgres' database and will use the first
+        # PostgreSQL database instead." warning.
+        with super(PostgresDatabaseWrapper, self)._nodb_cursor() as cursor:
+            yield cursor
+
     @cached_property
     def cockroachdb_server_info(self):
         # Something like 'CockroachDB CCL v20.1.0-alpha.20191118-1842-g60d40b8
