@@ -7,13 +7,6 @@ from django.db.models import ForeignKey
 
 
 class DatabaseSchemaEditor(PostgresDatabaseSchemaEditor):
-    @property
-    def sql_create_index(self):
-        if self.connection.features.is_cockroachdb_20_2:
-            return PostgresDatabaseSchemaEditor.sql_create_index
-        else:
-            return 'CREATE INDEX %(name)s ON %(table)s%(using)s (%(columns)s)%(extra)s'
-
     # The PostgreSQL backend uses "SET CONSTRAINTS ... IMMEDIATE" before
     # "ALTER TABLE..." to run any any deferred checks to allow dropping the
     # foreign key in the same transaction. This doesn't apply to cockroachdb.
@@ -23,17 +16,19 @@ class DatabaseSchemaEditor(PostgresDatabaseSchemaEditor):
     # constraints; must use this instead.
     sql_delete_unique = "DROP INDEX %(name)s CASCADE"
 
-    # adding a REFERENCES constraint while also adding a column via ALTER not
-    # supported: https://github.com/cockroachdb/cockroach/issues/32917
-    @property
-    def sql_create_column_inline_fk(self):
-        if self.connection.features.is_cockroachdb_20_2:
-            # The PostgreSQL backend uses "SET CONSTRAINTS ... IMMEDIATE"
-            # after creating this foreign key. This isn't supported by
-            # CockroachDB.
-            return 'CONSTRAINT %(name)s REFERENCES %(to_table)s(%(to_column)s)%(deferrable)s'
-        else:
+    # The PostgreSQL backend uses "SET CONSTRAINTS ... IMMEDIATE" after
+    # creating this foreign key. This isn't supported by CockroachDB.
+    sql_create_column_inline_fk = 'CONSTRAINT %(name)s REFERENCES %(to_table)s(%(to_column)s)%(deferrable)s'
+
+    def add_index(self, model, index, concurrently=False):
+        if index.contains_expressions and not self.connection.features.supports_expression_indexes:
             return None
+        super().add_index(model, index, concurrently)
+
+    def remove_index(self, model, index, concurrently=False):
+        if index.contains_expressions and not self.connection.features.supports_expression_indexes:
+            return None
+        super().remove_index(model, index, concurrently)
 
     def _index_columns(self, table, columns, col_suffixes, opclasses):
         # cockroachdb doesn't support PostgreSQL opclasses.
